/* 
 * Copyright (C) 2006 Johannes Hausensteiner (johannes.hausensteiner@pcl.at)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/* 
 * Filename: cmd.c
 *
 * Function: command module
 *
 * Part of:  DIY Calculator loader application
 *
 * Changelog
 *
 *  0.1  16.Oct.2006   JH   new
 *  1.1  14.Nov.2006   JH   code review
 *
 */


#include <gtk/gtk.h>

#include <string.h>

#ifndef WIN32
#include <unistd.h>          /* usleep() */
#endif

#include "diy_load.h"
#include "serial.h"
#include "cmd.h"
#include "gtk_msgbox.h"
#include "gtk_helper.h"


/******************************  Defines  *********************************/


/******************************  Macros  **********************************/
#define DEBUG(x)  if (debug) g_print x  /* a simple trace macro */

/******************************  Structs  *********************************/


/**************************  Private Functions  ***************************/


/************************  Function Prototypes  ***************************/


/******************************  Globals  *********************************/
static const char *cmd_names[] =
{
  "help",
  "info",
  "md",
  "mw",
  "load",
  "step",
  "run",
  "reset",
  "cpflash",
  "cpram",
  "eflash",
  "rdump",
  NULL
};

extern int debug;


/* send_cmd */
/**************************************************************************/
int send_cmd (               /* send a command to the DIY Calculator      */
  int cmd)                   /* the command                               */
/***************************************************************************
* globals     : cmd_names[]                                                *
* called from : disp_cpu_reg(), load_ram(), init_port(), goto_stack(),     *
*               goto_pc(), do_version_info()                               *
* calls       : write_port()                                               *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Send a command to the DIY Calculator.                                    *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* 0 .. OK                                                                  *
***************************************************************************/
{
  char buf[BUFLEN];

  DEBUG (("send_cmd(): %s\n", cmd_names[cmd]));
  strcpy (buf, cmd_names[cmd]);
  strcat (buf, "\n");
  return write_port (buf, strlen(buf));
}

/* wait_for_string */
/**************************************************************************/
int wait_for_string (        /* wait for a string form the DIY Calculator */
  const char *s)             /* the string we are waiting for             */
/***************************************************************************
* globals     :                                                            *
* called from : wait_for_prompt(), load_ram()                              *
* calls       : read_port()                                                *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Wait for a given string outputted by the DIY Calculator.                 *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* 0 .. OK                                                                  *
* 1 .. timeout                                                             *
***************************************************************************/
{
  int  i, timer, ret;
  char buf[BUFLEN];

  DEBUG (("wait_for_string(): s=\"%s\"", s));
  ret = 0;
  timer = TIMEOUT / 16;
  i = 0;
  while (--timer)
  {
    i = read_port (buf, BUFLEN);
    if (i == -1)
      return 1;
    buf[i] = '\0';
    usleep (1000);
//    DEBUG (("; buf=\"%s\"", buf));
    if (strstr (buf, s) != NULL)
      break;
  }
  if (timer == 0)
    ret = 1;

  DEBUG (("\n"));
  return ret;
}

/* wait_for_prompt */
/**************************************************************************/
int wait_for_prompt ()       /* wait for the DIY Calculator prompt        */
/***************************************************************************
* globals     :                                                            *
* called from : do_connect(), exec_term(), do_reset(), read_reg()          *
* calls       : wait_for_string()                                          *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Wait for the DIY Calculator outputting a prompt.                         *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* 0 .. OK                                                                  *
* 1 .. timeout                                                             *
***************************************************************************/
{
  return wait_for_string ("# ");
}
