/* 
 * Copyright (C) 2006 Johannes Hausensteiner (johannes.hausensteiner@pcl.at)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/* 
 * Filename: diy_load.h
 *
 * Function: common definitions for DIY Calculator loader application
 *
 * Changelog
 *
 *  0.1  20.Oct.2006   JH   new
 *  0.2  23.Oct.2006   JH   
 *  0.9  24.Oct.2006   JH   new design
 *  1.0  10.Nov.2006   JH   first delivered version
 *  1.1  14.Nov.2006   JH   code review
 *
 */


#ifndef __DIY_LOAD_H
#define __DIY_LOAD_H

#define VER "1.1"

#ifdef WIN32
#include <windows.h>
#endif

/******************************  Defines  *********************************/
#define PROGNAME  "DIY Calculator Loader Utility"

#define OK            0
#define BUFLEN      200
#define PROMPT_CHAR '#'
#define TIMEOUT    1000

#ifdef WIN32
#define TERMINAL_FONT "monospace 10"
#define SCROLL_SIZE   15
#else
#define TERMINAL_FONT "monospace 12"
#define SCROLL_SIZE    17
#endif


/* the DIY Calculator commands */
enum CMD
{
  HELP,
  INFO,
  MD,
  MW,
  LOAD,
  STEP,
  RUN,
  RESET,
  CPFLASH,
  CPRAM,
  EFLASH,
  DUMPREG,
  NO_OF_CMDS
};

/* control characters */
#define CTRL_A     '\001'    /* step cmd                                  */
#define CTRL_B     '\002'    /* run cmd                                   */
#define CTRL_C     '\003'    /* (soft-) reset cmd                         */


/******************************  Macros  **********************************/
#ifdef WIN32
#define strcasecmp(s1, s2) stricmp(s1, s2)
#define usleep(usec) Sleep(usec/1000)
#endif


/******************************  Structs  *********************************/


/************************  Function Prototypes  ***************************/
int init_port (char *name);  /* wrapper for open_port()                   */
int atox (char *s);          /* convert string to HEX number              */


/******************************  Globals  *********************************/



#endif  /* __DIY_LOAD_H */

