/*-!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!
! Filename: gtk_filename.c Function: wrapper for File Open dialog          !
! Module  :                                                                !
!                                                                          !
!   Vers     Date    Auth  Changes                                         !
! ------------------------------------------------------------------------ !
!   0.1  13.Oct.2005  JH   Created                                         !
!        24.Oct.2005  JH   added native Windows FileOpen dialog            !
!        27.Oct.2005  JH   GetOpenFileName / GetSaveFileName               !
!        24.Nov.2005  JH   Error 12261: set OFN_NOCHANGEDIR in OFN struct  !
!                          return lpstrFile instead of lpstrFileTitle      !
!                                                                          !
!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-*/


#include <gtk/gtk.h>         /* GTK stuff */

#ifdef WIN32
#include <windows.h>         /* Windows stuff */
#endif

#include <string.h>          /* strcpy(), strcat(), strchr() */
#include <stdio.h>

#include "gtk_filename.h"    /* the prototypes */
#include "gtk_multibox.h"

/******************************  Constants  *******************************/
#ifdef WIN32
#define USE_WIN_DIALOG   1   /* 1: use Windows native FileOpen dialog     */
#else
#define USE_WIN_DIALOG   0   /* always 0 when not in Windows ...          */
#endif

/********************************  Macros  ********************************/
#define debug(x)  /* printf x */

/***************************  Typedefs, Structs  **************************/

/***************************  Private Functions  **************************/

/***************************  Function Prototypes  ************************/
void file_selection_close (  /* callback func for file selection dialog   */
  GtkWidget  *window,        /* the window                                */
  gpointer   data            /*                                           */
);

void file_selection_callback ( /* callback func for file selection dialog */
  GtkWidget  *window,        /* the window                                */
  gpointer   data            /*                                           */
);


/*******************************  Globals  ********************************/
static int cancel;           /* for handshake                             */
static char local_filename[PATH_MAX];  /* for handshake                   */


/* gtk_filename */
/**************************************************************************/
char *gtk_filename (         /* display a file selection dialog           */
  GtkWidget *parent,         /* parent window                             */
  unsigned int flags)        /* operational flags                         */
/***************************************************************************
* globals     : cancel, local_filename                                     *
* called from : config_callback()                                          *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Display a file selection dialog and select (or cancel) a file.           *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* filename ..OK                                                            *
* NULL .. else                                                             *
***************************************************************************/
{
#if USE_WIN_DIALOG
  OPENFILENAME OpenFileNameStruct;
  char *FileNameFilter = {
    "RAM Files (*.ram)\0*.ram\0All Files (*.*)\0*.*\0\0"};
  char *p;

  cancel = 0;

  p = "";
  if (flags & FN_LOAD)
    p = "Load File";
  else if (flags & FN_SAVE)
    p = "Save File";
  else if (flags & FN_SAVE_AS)
    p = "Save File As";

  *local_filename = '\0';
  OpenFileNameStruct.lStructSize = sizeof (OpenFileNameStruct);
  OpenFileNameStruct.hwndOwner = NULL;
  OpenFileNameStruct.hInstance = NULL;
  OpenFileNameStruct.lpstrFilter = FileNameFilter;
  OpenFileNameStruct.lpstrCustomFilter = NULL;
  OpenFileNameStruct.nMaxCustFilter = 0;
  OpenFileNameStruct.nFilterIndex = 1;
  OpenFileNameStruct.lpstrFile = local_filename;
  OpenFileNameStruct.nMaxFile = PATH_MAX;
  OpenFileNameStruct.lpstrFileTitle = NULL;
  OpenFileNameStruct.nMaxFileTitle = PATH_MAX;
  OpenFileNameStruct.lpstrInitialDir = NULL;
  OpenFileNameStruct.lpstrTitle = p;
  OpenFileNameStruct.Flags = OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
  OpenFileNameStruct.nFileOffset = 0;
  OpenFileNameStruct.nFileExtension = 0;
  OpenFileNameStruct.lpstrDefExt = "txt";
  OpenFileNameStruct.lCustData = 0;
  OpenFileNameStruct.lpfnHook = NULL;
  OpenFileNameStruct.lpTemplateName = NULL;
  if (flags & FN_LOAD)
    cancel = !GetOpenFileName (&OpenFileNameStruct);
  else
    cancel = !GetSaveFileName (&OpenFileNameStruct);

  p = local_filename;

  if (cancel)
    p = NULL;

  debug (("gtk_filename() returns:%s\n", local_filename));
  return p;
#else
  GtkWidget *file_selector;
  char *p;

  cancel = 0;

  p = "";
  if (flags & FN_LOAD)
    p = "Load File";
  else if (flags & FN_SAVE)
    p = "Save File";
  else if (flags & FN_SAVE_AS)
    p = "Save File As";

  /* create the file selection dialog */
  file_selector = gtk_file_selection_new (p);
  /* we want it to be a modal window */
  gtk_window_set_modal (GTK_WINDOW(file_selector), TRUE);
  /* ... and it should be on top of our main window */
  gtk_window_set_transient_for(GTK_WINDOW(file_selector),GTK_WINDOW(parent));

  /* connect the signals */
  g_signal_connect (G_OBJECT (file_selector),
    "destroy", G_CALLBACK(gtk_widget_destroy), file_selector);

  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
    "clicked", G_CALLBACK (file_selection_callback), file_selector);

  g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),
    "clicked", G_CALLBACK (file_selection_close), file_selector);

  if (!(flags & FN_BUTTONS))
    gtk_file_selection_hide_fileop_buttons (
      GTK_FILE_SELECTION(file_selector));

  gtk_widget_show (file_selector);
  gtk_main ();

  p = local_filename;

  if (cancel)
    p = NULL;

  debug (("gtk_filename() returns:%s\n", local_filename));
  return p;
#endif
}
#if !USE_WIN_DIALOG
/* file_selection_close */
/**************************************************************************/
void file_selection_close (  /* callback func for file selection dialog   */
  GtkWidget  *window,        /* the window                                */
  gpointer   data)           /*                                           */
/***************************************************************************
* globals     : cancel                                                     *
* called from : callback                                                   *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* This is the callback function for the file selection dialog, CANCEL      *
* button.                                                                  *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* void                                                                     *
***************************************************************************/
{
  debug (("file_selection_close(): window=0x%8.8x, data=0x%8.8x\n",
    (unsigned)window, (unsigned)data));
  cancel = 1;
  gtk_widget_destroy (GTK_WIDGET(data));
  gtk_main_quit ();
}

/* file_selection_callback */
/**************************************************************************/
void file_selection_callback ( /* callback func for file selection dialog */
  GtkWidget  *window,        /* the window                                */
  gpointer   data)           /*                                           */
/***************************************************************************
* globals     : local_filename                                             *
* called from : callback                                                   *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* This is the callback function for the file selection dialog, OK button.  *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* void                                                                     *
***************************************************************************/
{
  GtkFileSelection *fs;

  debug (("file_selection_callback(): window=0x%8.8x, data=0x%8.8x\n",
    (unsigned)window, (unsigned)data));
  fs = (GtkFileSelection *) data;
  strcpy (local_filename, gtk_file_selection_get_filename (fs));
  gtk_widget_destroy (GTK_WIDGET(fs));
  gtk_main_quit ();
}
#endif
/* gtk_file_overwrite */
/**************************************************************************/
int gtk_file_overwrite (     /* display a file-overwrite dialog box       */
  GtkWidget *parent,         /* parent window                             */
  char *filename)            /* the filename of the file                  */
/***************************************************************************
* globals     :                                                            *
* called from : config_callback()                                          *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Diaplay a "Confirm File Overwrite" dialog box.                           *
*                                                                          *
* Returns                                                                  *
* --------                                                                 *
* 0 .. NO                                                                  *
* 1 .. YES                                                                 *
***************************************************************************/
{
  char buf[100];

  sprintf (buf, "File \"%s\" exists. Overwrite?", filename);
  return !gtk_multi_box (parent, "Confirm Overwrite", buf, 2, "YES", "NO");
}
