/*-!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!
! Filename: gtk_msgbox.c  Function: A Message Box object for GTK           !
! Part of:                                                                 !
!                                                                          !
!                                                                          !
! Vers.   Date    Auth.   Changes                                          !
! ------------------------------------------------------------------------ !
! 0.1    3.Mar.05  JH     Created                                          !
!       23.Mar.05  JH     DOS/Windows/Linux GUI/Console all one project    !
!     15.Oct.2005  JH     msgbox is modal and transient                    !
!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-*/


#include <gtk/gtk.h>
#include <stdarg.h>

#include "gtk_msgbox.h"


/******************************  Defines  *********************************/
#ifndef ABS
#define ABS(x)    ((x)<0?(x):-(x))
#endif

/******************************  Typedefs *********************************/

/**************************  Private Functions  ***************************/
static void msgbox_close (   /* destroy the specified widget              */
  GtkWidget *widget,         /* the parent widget                         */
  gpointer  data             /* the widget to destroy                     */
);

/******************************  Globals  *********************************/
static GtkWidget *msgbox;    /* the message box                           */


/* gtk_msg_box */
/**************************************************************************/
void gtk_msg_box (           /* the message box main function             */
  GtkWidget *parent,         /* the parent window                         */
  char *title,               /* the title of the message box              */
  char *text,                /* the text to be displayed in the msg box   */
  GtkJustification jtype)    /* the justification type for `text'         */
/***************************************************************************
* globals     : msgbox                                                     *
* called from :                                                            *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* This is the Message Box object main function.                            *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* index of clicked button                                                  *
***************************************************************************/
{
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *overall_box;

  /* create the message box */
  msgbox = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  /* we want it to be a modal window */
  gtk_window_set_modal (GTK_WINDOW(msgbox), TRUE);
  /* ... and it should be on top of our main window */
  gtk_window_set_transient_for(GTK_WINDOW(msgbox),GTK_WINDOW(parent));
  gtk_window_set_title (GTK_WINDOW (msgbox), title);
  gtk_container_set_border_width (GTK_CONTAINER (msgbox), 20);

  overall_box = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (msgbox), overall_box);
  gtk_container_border_width (GTK_CONTAINER (msgbox), 10);

  /* put the text */
  label = gtk_label_new (text);
  gtk_label_set_justify (GTK_LABEL(label), jtype);
  gtk_box_pack_start (GTK_BOX (overall_box), label,TRUE,FALSE,0);

  /* the OK button */
  button = gtk_button_new_with_label ("OK");
  g_signal_connect (G_OBJECT(button),"clicked",G_CALLBACK(msgbox_close),NULL);

  gtk_box_pack_start (GTK_BOX (overall_box),button,TRUE,FALSE,0);

  gtk_window_set_modal (GTK_WINDOW (msgbox), TRUE);
  gtk_window_set_focus (GTK_WINDOW (msgbox), button);

  gtk_window_set_position (GTK_WINDOW(msgbox),GTK_WIN_POS_CENTER_ON_PARENT);

  gtk_widget_show_all (msgbox);

  gtk_main();
}

/* msgbox_close */
/**************************************************************************/
void msgbox_close (          /* destroy the specified widget              */
  GtkWidget *widget,         /* the parent widget                         */
  gpointer  data)            /* the widget to destroy                     */
/***************************************************************************
* globals     : msgbox                                                     *
* called from : callback function                                          *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Destroy the specified widget.                                            *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* void                                                                     *
***************************************************************************/
{
  gtk_widget_destroy (msgbox);
  gtk_main_quit ();
}

