/* 
 * Copyright (C) 2006 Johannes Hausensteiner (johannes.hausensteiner@pcl.at)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/* 
 * Filename: serial.h
 *
 * Function: interface to serial.c
 *
 * Part of:  DIY Calculator loader application
 *
 * Changelog
 *
 *  0.1  13.Oct.2006   JH   new
 *  0.2  24.Oct.2006   JH   new design
 *  1.0  10.Nov.2006   JH   first delivered version
 *  1.1  14.Nov.2006   JH   code review
 *
 */


#ifndef __SERIAL_H
#define __SERIAL_H

/******************************  Defines  *********************************/
#define MAX_COMM_PORTS    20

#define RXBUF_SIZE     2048

/******************************  Macros  **********************************/

/******************************  Structs  *********************************/
struct COMM_PORT
{
  char name[20];             /* name string of the communications port    */
  int  number;               /* communications port number                */
  int  is_virtual;           /* 0..real hardware, 1..virtual comm. port   */
};

/***************************  Function Prototypes  ************************/
int get_avail_ports (void);  /* find the available communication ports    */

int open_port (              /* open a specified communcations port       */
  char *portname             /* which port to open                        */
);

int close_port (             /* close a specified communcations port      */
  int fd                     /* which port to close                       */
);

int write_port (
  void *buf,                 /* the buffer                                */
  size_t count               /* length of the buffer                      */
);

int read_port (
  void *buf,                 /* the buffer                                */
  size_t count               /* length of the buffer                      */
);

int read_line (
  void *buf,                 /* the buffer                                */
  size_t count               /* length of the buffer                      */
);

int char_avail (             /* check for something available             */
  int fd                     /* the communications port file descriptor   */
);
  
#endif /* __SERIAL_H */
