/* 
 * Copyright (C) 2006 Johannes Hausensteiner (johannes.hausensteiner@pcl.at)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
/* 
 * Filename: set_comm_port.c
 *
 * Function: Setup Communications Port dialog
 *
 * Part of:  DIY Calculator loader application
 *
 * Changelog
 *
 *  0.1  21.Oct.2006   JH   new
 *  1.0  11.Nov.2006   JH   first delivered version
 *  1.1  14.Nov.2006   JH   code review
 *
 */


#include <gtk/gtk.h>


#include <string.h>

#include "diy_load.h"
#include "serial.h"          /* struct COMM_PORT */

/******************************  Defines  *********************************/


/******************************  Macros  **********************************/
#define DEBUG(x)  if (debug) g_print x  /* a simple trace macro */

/******************************  Structs  *********************************/


/**************************  Private Functions  ***************************/
static void set_comm_port_close (GtkWidget *widget, gpointer data);

/******************************  Globals  *********************************/
static GtkWidget *window;

extern char portname[];
extern struct COMM_PORT portlist[];
extern int no_of_ports;
extern int debug;
extern int offline;
extern int portfd;


/* set_comm_port_close */
/**************************************************************************/
void set_comm_port_close (   /* a new comm. port is selected              */
  GtkWidget *widget,         /* the ok or cancel button                   */
  gpointer data)             /* the combo box                             */
/***************************************************************************
* globals     : window, offline, portname, portlist[], portfd              *
* called from :                                                            *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Is called when the user clicks OK or Cancel in the "Setup Communications *
* Port" dialog.                                                            *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* void                                                                     *
***************************************************************************/
{
  int portindex;

  DEBUG (("set_comm_port_close(): "));
  if (!offline && data)
  {
    portindex = gtk_combo_box_get_active (GTK_COMBO_BOX(data));
    gtk_widget_destroy (GTK_WIDGET(window));
    strcpy (portname, portlist[portindex].name);
    DEBUG (("new port: %s, %d%s\n", portname, portindex,
      portlist[portindex].is_virtual?", virtual":""));
    /* close old port */
    close_port (portfd);
    /* open new port */
    portfd = init_port (portname);
  } else
  {
    DEBUG (("Cancel\n"));
    gtk_widget_destroy (GTK_WIDGET(window));
  }
}

/* do_set_comm_port */
/**************************************************************************/
int do_set_comm_port (       /* do the Setup Communications Port dialog   */
  GtkWidget *parent)         /* the parent window                         */
/***************************************************************************
* globals     : window, portlist[], no_of_ports, portname                  *
* called from : set_comm_port()                                            *
* calls       :                                                            *
*                                                                          *
* Function                                                                 *
* --------                                                                 *
* Define the used communications port.                                     *
*                                                                          *
* Returns                                                                  *
* -------                                                                  *
* void                                                                     *
***************************************************************************/
{
  GtkWidget *table, *label, *combo_box, *button;
  int i;

  DEBUG (("do_set_comm_port(): "));
  for (i=0; i<no_of_ports; i++)
  {
    if (strcasecmp(portlist[i].name, portname) == 0)
      break;
  }

  DEBUG (("%s, %d%s\n",
    portlist[i].name, i, portlist[i].is_virtual?", virtual":""));

  /* create a new top level window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal (GTK_WINDOW(window), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(window),
    GTK_WINDOW(parent));
  gtk_window_set_title (GTK_WINDOW (window),
    "Select Communications Port");
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);

  table = gtk_table_new (2, 2, TRUE);
  gtk_table_set_row_spacings (GTK_TABLE(table), 10);
  gtk_table_set_col_spacings (GTK_TABLE(table), 10);
  gtk_container_add (GTK_CONTAINER(window), table);
  
  label = gtk_label_new ("Select port:");
  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
  
  combo_box = gtk_combo_box_new_text ();
  for (i=0; i<no_of_ports; i++)
  {
    gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), portlist[i].name);
    if (strcasecmp (portlist[i].name, portname) == 0)
      gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), i);
  }
  gtk_table_attach_defaults (GTK_TABLE (table), combo_box, 1, 2, 0, 1);
  
  /* OK and Cancel buttons */
  button = gtk_button_new_with_label ("OK");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (set_comm_port_close), combo_box);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 1, 2);

  button = gtk_button_new_with_label ("Cancel");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (set_comm_port_close), NULL);
  gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 1, 2);

  /* put the dialog box to the center of the main window */
  gtk_window_set_position (GTK_WINDOW (window),
    GTK_WIN_POS_CENTER_ON_PARENT);

  gtk_widget_show_all (window);

  return portfd;
}
